@@ -8,11 +8,21 @@ class ResidualVQ(nn.Module):
         self,
         *,
         num_quantizers,
+        shared_codebook = False,
         **kwargs
     ):
         super().__init__()
         self.layers = nn.ModuleList([VectorQuantize(**kwargs) for _ in range(num_quantizers)])
 
+        if not shared_codebook:
+            return
+
+        first_vq, *rest_vq = self.layers
+        codebook = first_vq._codebook.embed
+
+        for vq in rest_vq:
+            vq._codebook.embed = codebook
+
     def forward(self, x):
         quantized_out = 0.
         residual = x
